@@ -22,7 +22,6 @@ import tempfile
 
 import extras
 import fixtures
-from keystoneclient import discover as ks_discover
 import mock
 from oslo_config import cfg
 import testtools
@@ -30,6 +29,7 @@ from testtools import matchers
 
 from os_collect_config import cache
 from os_collect_config import collect
+from os_collect_config import config_drive
 from os_collect_config import exc
 from os_collect_config.tests import test_cfn
 from os_collect_config.tests import test_ec2
@@ -74,16 +74,20 @@ class TestCollect(testtools.TestCase):
             'cfn': {'requests_impl': test_cfn.FakeRequests(self)},
             'heat': {
                 'keystoneclient': test_heat.FakeKeystoneClient(self),
-                'heatclient': test_heat.FakeHeatClient(self)
+                'heatclient': test_heat.FakeHeatClient(self),
+                'discover_class': test_heat.FakeKeystoneDiscover
             },
             'request': {'requests_impl': test_request.FakeRequests},
             'zaqar': {
                 'keystoneclient': test_zaqar.FakeKeystoneClient(self),
-                'zaqarclient': test_zaqar.FakeZaqarClient(self)
+                'zaqarclient': test_zaqar.FakeZaqarClient(self),
+                'discover_class': test_heat.FakeKeystoneDiscover
             },
         }
-        return collect.__main__(args=fake_args,
-                                collector_kwargs_map=collector_kwargs_map)
+        with mock.patch.object(config_drive, 'get_metadata') as gm:
+            gm.return_value = {}
+            return collect.__main__(args=fake_args,
+                                    collector_kwargs_map=collector_kwargs_map)
 
     def _fake_popen_call_main(self, occ_args):
         calls = []
@@ -111,7 +115,7 @@ class TestCollect(testtools.TestCase):
             '--config-file',
             '/dev/null',
             '--cfn-metadata-url',
-            'http://127.0.0.1:8000/v1/',
+            'http://192.0.2.1:8000/v1/',
             '--cfn-stack-name',
             'foo',
             '--cfn-path',
@@ -129,7 +133,7 @@ class TestCollect(testtools.TestCase):
             '--heat-project-id',
             '9f6b09df-4d7f-4a33-8ec3-9924d8f46f10',
             '--heat-auth-url',
-            'http://127.0.0.1:5000/v3',
+            'http://192.0.2.1:5000/v3',
             '--heat-stack-id',
             'a/c482680f-7238-403d-8f76-36acf0c8e0aa',
             '--heat-resource-name',
@@ -225,7 +229,7 @@ class TestCollect(testtools.TestCase):
             '--config-file',
             '/dev/null',
             '--cfn-metadata-url',
-            'http://127.0.0.1:8000/v1/',
+            'http://192.0.2.1:8000/v1/',
             '--cfn-stack-name',
             'foo',
             '--cfn-path',
@@ -275,7 +279,7 @@ class TestCollect(testtools.TestCase):
             '--config-file', '/dev/null',
             '--print',
             '--cfn-metadata-url',
-            'http://127.0.0.1:8000/v1/',
+            'http://192.0.2.1:8000/v1/',
             '--cfn-stack-name',
             'foo',
             '--cfn-path',
@@ -329,6 +333,7 @@ class TestCollect(testtools.TestCase):
 
 
 class TestCollectAll(testtools.TestCase):
+
     def setUp(self):
         super(TestCollectAll, self).setUp()
         self.log = self.useFixture(fixtures.FakeLogger())
@@ -343,52 +348,52 @@ class TestCollectAll(testtools.TestCase):
 
         cfg.CONF.cachedir = self.cache_dir.path
         cfg.CONF.backup_cachedir = self.backup_cache_dir.path
-        cfg.CONF.cfn.metadata_url = 'http://127.0.0.1:8000/v1/'
+        cfg.CONF.cfn.metadata_url = 'http://192.0.2.1:8000/v1/'
         cfg.CONF.cfn.stack_name = 'foo'
         cfg.CONF.cfn.path = ['foo.Metadata']
         cfg.CONF.cfn.access_key_id = '0123456789ABCDEF'
         cfg.CONF.cfn.secret_access_key = 'FEDCBA9876543210'
         cfg.CONF.heat_local.path = [_setup_heat_local_metadata(self)]
-        cfg.CONF.heat.auth_url = 'http://127.0.0.1:5000/v3'
+        cfg.CONF.heat.auth_url = 'http://192.0.2.1:5000/v3'
         cfg.CONF.heat.user_id = '0123456789ABCDEF'
         cfg.CONF.heat.password = 'FEDCBA9876543210'
         cfg.CONF.heat.project_id = '9f6b09df-4d7f-4a33-8ec3-9924d8f46f10'
         cfg.CONF.heat.stack_id = 'a/c482680f-7238-403d-8f76-36acf0c8e0aa'
         cfg.CONF.heat.resource_name = 'server'
         cfg.CONF.local.path = [_setup_local_metadata(self)]
-        cfg.CONF.request.metadata_url = 'http://127.0.0.1:8000/my_metadata/'
-        cfg.CONF.zaqar.auth_url = 'http://127.0.0.1:5000/v3'
+        cfg.CONF.request.metadata_url = 'http://192.0.2.1:8000/my_metadata/'
+        cfg.CONF.zaqar.auth_url = 'http://192.0.2.1:5000/v3'
         cfg.CONF.zaqar.user_id = '0123456789ABCDEF'
         cfg.CONF.zaqar.password = 'FEDCBA9876543210'
         cfg.CONF.zaqar.project_id = '9f6b09df-4d7f-4a33-8ec3-9924d8f46f10'
         cfg.CONF.zaqar.queue_id = '4f3f46d3-09f1-42a7-8c13-f91a5457192c'
 
-    @mock.patch.object(ks_discover.Discover, '__init__')
-    @mock.patch.object(ks_discover.Discover, 'url_for')
-    def _call_collect_all(self, mock_url_for, mock___init__, store,
-                          collector_kwargs_map=None, collectors=None):
-        mock___init__.return_value = None
-        mock_url_for.return_value = cfg.CONF.heat.auth_url
+    def _call_collect_all(self, store, collector_kwargs_map=None,
+                          collectors=None):
         if collector_kwargs_map is None:
             collector_kwargs_map = {
                 'ec2': {'requests_impl': test_ec2.FakeRequests},
                 'cfn': {'requests_impl': test_cfn.FakeRequests(self)},
                 'heat': {
                     'keystoneclient': test_heat.FakeKeystoneClient(self),
-                    'heatclient': test_heat.FakeHeatClient(self)
+                    'heatclient': test_heat.FakeHeatClient(self),
+                    'discover_class': test_heat.FakeKeystoneDiscover
                 },
                 'request': {'requests_impl': test_request.FakeRequests},
                 'zaqar': {
                     'keystoneclient': test_zaqar.FakeKeystoneClient(self),
-                    'zaqarclient': test_zaqar.FakeZaqarClient(self)
+                    'zaqarclient': test_zaqar.FakeZaqarClient(self),
+                    'discover_class': test_heat.FakeKeystoneDiscover
                 },
             }
         if collectors is None:
             collectors = cfg.CONF.collectors
-        return collect.collect_all(
-            collectors,
-            store=store,
-            collector_kwargs_map=collector_kwargs_map)
+        with mock.patch.object(config_drive, 'get_metadata') as gm:
+            gm.return_value = {}
+            return collect.collect_all(
+                collectors,
+                store=store,
+                collector_kwargs_map=collector_kwargs_map)
 
     def _test_collect_all_store(self, collector_kwargs_map=None,
                                 expected_changed=None):
@@ -413,12 +418,14 @@ class TestCollectAll(testtools.TestCase):
                 'requests_impl': test_cfn.FakeRequestsSoftwareConfig(self)},
             'heat': {
                 'keystoneclient': test_heat.FakeKeystoneClient(self),
-                'heatclient': test_heat.FakeHeatClient(self)
+                'heatclient': test_heat.FakeHeatClient(self),
+                'discover_class': test_heat.FakeKeystoneDiscover
             },
             'request': {'requests_impl': test_request.FakeRequests},
             'zaqar': {
                 'keystoneclient': test_zaqar.FakeKeystoneClient(self),
-                'zaqarclient': test_zaqar.FakeZaqarClient(self)
+                'zaqarclient': test_zaqar.FakeZaqarClient(self),
+                'discover_class': test_heat.FakeKeystoneDiscover
             },
         }
         expected_changed = set((
@@ -456,12 +463,14 @@ class TestCollectAll(testtools.TestCase):
                 'requests_impl': test_cfn.FakeRequestsSoftwareConfig(self)},
             'heat': {
                 'keystoneclient': test_heat.FakeKeystoneClient(self),
-                'heatclient': test_heat.FakeHeatClient(self)
+                'heatclient': test_heat.FakeHeatClient(self),
+                'discover_class': test_heat.FakeKeystoneDiscover
             },
             'request': {'requests_impl': test_request.FakeRequests},
             'zaqar': {
                 'keystoneclient': test_zaqar.FakeKeystoneClient(self),
-                'zaqarclient': test_zaqar.FakeZaqarClient(self)
+                'zaqarclient': test_zaqar.FakeZaqarClient(self),
+                'discover_class': test_heat.FakeKeystoneDiscover
             },
         }
         (changed_keys, paths) = self._call_collect_all(
@@ -474,9 +483,16 @@ class TestCollectAll(testtools.TestCase):
         # Commit
         for changed in changed_keys:
             cache.commit(changed)
+
+        # Replace the ec2 requests with a failing one to simulate a transient
+        # network failure
+        soft_config_map['ec2'] = {'requests_impl': test_ec2.FakeFailRequests}
         (changed_keys, paths2) = self._call_collect_all(
             store=True, collector_kwargs_map=soft_config_map)
         self.assertEqual(set(), changed_keys)
+
+        # check the second collect includes cached ec2 data despite network
+        # failure
         self.assertEqual(paths, paths2)
 
     def test_collect_all_nostore(self):
@@ -493,7 +509,8 @@ class TestCollectAll(testtools.TestCase):
             'cfn': {'requests_impl': test_cfn.FakeRequests(self)}
         }
         (changed_keys, content) = self._call_collect_all(
-            store=False, collector_kwargs_map=collector_kwargs_map)
+            store=False, collector_kwargs_map=collector_kwargs_map,
+            collectors=['ec2', 'cfn'])
         self.assertEqual(set(), changed_keys)
         self.assertThat(content, matchers.IsInstance(dict))
         self.assertNotIn('ec2', content)
